# -*- coding: utf-8 -*-

# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License

"""Langchain related widgets."""

from .status import LangchainStatusWidget

__all__ = [
    "LangchainStatusWidget",
]
